// src/extension.ts
import * as Main from "resource:///org/gnome/shell/ui/main.js";

// src/utils.ts
import Gio from "gi://Gio";
function exec(argv, input = null, cancellable = null) {
  let flags = Gio.SubprocessFlags.STDOUT_PIPE;
  if (input !== null) flags |= Gio.SubprocessFlags.STDIN_PIPE;
  const proc = new Gio.Subprocess({
    argv,
    flags
  });
  proc.init(cancellable);
  return new Promise((resolve, reject) => {
    proc.communicate_utf8_async(input, cancellable, (proc2, res) => {
      try {
        resolve(proc2.communicate_utf8_finish(res)[1]);
      } catch (e) {
        reject(e);
      }
    });
  });
}
function titleCase(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}

// src/extension.ts
import GObject from "gi://GObject";
import Clutter from "gi://Clutter";
import GLib from "gi://GLib";
import St from "gi://St";
import {
  PopupMenuSection,
  PopupMenuItem,
  Ornament
} from "resource:///org/gnome/shell/ui/popupMenu.js";
import {
  QuickMenuToggle,
  SystemIndicator
} from "resource:///org/gnome/shell/ui/quickSettings.js";
import {
  Extension,
  gettext as _
} from "resource:///org/gnome/shell/extensions/extension.js";
var QuickSettingsMenu = GObject.registerClass(
  class QuickSettingsMenu2 extends QuickMenuToggle {
    _section = null;
    _items = /* @__PURE__ */ new Map();
    _extension = null;
    _init() {
      super._init({
        title: _("Fan Control"),
        iconName: "weather-tornado-symbolic",
        toggleMode: true
      });
      this.menu.setHeader(
        "weather-tornado-symbolic",
        _("Framework Fan Control")
      );
      this.connect("clicked", async () => {
        await exec(["fw-fanctrl", this?.checked ? "resume" : "pause"]);
        await this._extension?._sync();
      });
      const settingsButton = new St.Button({
        style_class: "fw-fctrl-preferences-button",
        y_align: Clutter.ActorAlign.CENTER,
        x_align: Clutter.ActorAlign.END,
        x_expand: true,
        child: new St.Icon({
          icon_name: "applications-system-symbolic",
          icon_size: 14
        })
      });
      this.menu.addHeaderSuffix(settingsButton);
      settingsButton.connect("clicked", () => {
        this._extension?.openPreferences();
      });
      this.connect("destroy", () => {
        this._section?.destroy();
        this._section = null;
        this._extension = null;
        settingsButton.destroy();
        for (const item of this._items.values()) {
          item.destroy();
        }
        this._items.clear();
      });
    }
    async _setup(extension) {
      this._extension = extension;
      const result = await exec([
        "fw-fanctrl",
        "--output-format=JSON",
        "print",
        "list"
      ]);
      const { strategies } = JSON.parse(result);
      this._section = new PopupMenuSection();
      for (const strategy of strategies) {
        const item = new PopupMenuItem(titleCase(strategy));
        this._section?.addMenuItem(item);
        this._items?.set(strategy, item);
        item.connect("activate", async () => {
          await exec(["fw-fanctrl", "use", strategy]);
          await extension._sync();
        });
      }
      this.menu.addMenuItem(this._section);
    }
    _setFanState(state) {
      this.menu.setHeader(
        "weather-tornado-symbolic",
        _("Framework Fan Control"),
        `Temperature: ${state.temperature}\xB0C Fan Speed ${state.speed}%`
      );
      this.subtitle = `${state.temperature}\xB0C ${state.speed}%`;
      this.set_checked(state.active);
      for (const [itemStrategy, menuItem] of this._items.entries()) {
        menuItem.setOrnament(
          itemStrategy === state.strategy ? Ornament.CHECK : Ornament.NONE
        );
      }
    }
  }
);
var FwFanCtrl = class extends Extension {
  _indicator = null;
  _menu = null;
  _sourceId = null;
  _settings = null;
  enable() {
    this._settings = this.getSettings();
    this._menu = new QuickSettingsMenu();
    this._indicator = new SystemIndicator();
    this._indicator.quickSettingsItems.push(this._menu);
    Main.panel.statusArea.quickSettings.addExternalIndicator(
      this._indicator
    );
    this._createLoop(this._settings.get_int("refresh-interval"));
    this._settings.connect("changed::refresh-interval", (settings, key) => {
      this._createLoop(settings.get_int("refresh-interval"));
    });
    this._sync();
  }
  disable() {
    this._indicator?.destroy();
    this._indicator = null;
    this._menu?.destroy();
    this._menu = null;
    this._settings = null;
    if (this._sourceId) {
      GLib.Source.remove(this._sourceId);
      this._sourceId = null;
    }
  }
  _createLoop(interval) {
    if (this._sourceId) {
      GLib.Source.remove(this._sourceId);
      this._sourceId = null;
    }
    this._sourceId = GLib.timeout_add_seconds(
      GLib.PRIORITY_DEFAULT,
      interval,
      () => {
        this._sync();
        return GLib.SOURCE_CONTINUE;
      }
    );
  }
  async _sync() {
    if (this._menu && this._menu._items.size == 0) {
      await this._menu._setup(this);
    }
    try {
      const result = await exec([
        "fw-fanctrl",
        "--output-format=JSON",
        "print",
        "all"
      ]);
      const state = JSON.parse(result);
      this._menu?._setFanState(state);
      this._indicator.visible = state.active;
    } catch (error) {
      console.error("[fw-fanctrl-revived] Error checking state", error);
    }
  }
};
export {
  FwFanCtrl as default
};
